#!/bin/sh
#
# This file is part of Rheolef.
#
# Copyright (C) 2000-2009 Pierre Saramito 
#
# Rheolef is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Rheolef is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Rheolef; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# -------------------------------------------------------------------------
#
# make a distrib tree structure from all .deb
# see :
#   https://gna.org/cookbook/?func=detailitem&comingfrom=876&item_id=118
#
# TOD0: cutomize via command line args
# (ARCH=$build_cpu in script configure & Makefile)
DIST=${1-"debian"}
VARIANT=${2-"stable"}
ARCH=${3-"i386"}

if test -f VERSION; then
  true
else
  echo "$0: file VERSION not found." 1>&2
  exit 1
fi
VERSION=`cat VERSION`
topdir=`pwd`

# ---------------------------
# binaries
# ---------------------------
pkglist="
$topdir/../librheolef-doc_${VERSION}-1_$ARCH.deb
$topdir/../librheolef${VERSION}_${VERSION}-1_$ARCH.deb
$topdir/../rheolef-runtime_${VERSION}-1_$ARCH.deb
$topdir/../librheolef-dev_${VERSION}-1_$ARCH.deb
"

tmpdir="tmp-$$"
wrapdir="$tmpdir/$DIST"
destdir=$wrapdir/$VARIANT/$VERSION
rm -rf $tmpdir
mkdir -p $destdir
cp -rp $pkglist $destdir

cd $wrapdir
apt-ftparchive packages $VARIANT/$VERSION/ > $VARIANT/Packages
gzip -9 $VARIANT/Packages
# ---------------------------
# sources
# ---------------------------
pkglist="
$topdir/../rheolef_${VERSION}-1.dsc
$topdir/../rheolef_${VERSION}-1_${ARCH}.changes
$topdir/../rheolef_${VERSION}-1.tar.gz
"
cd ..
mkdir source
cp -rp $pkglist source
cd source
apt-ftparchive sources ./ > ./Sources
gzip -9 ./Sources
# ---------------------------
# make a tarball with all
# ---------------------------
cd ..
debdist="rheolef-$VERSION-$DIST-$VARIANT-$ARCH.tgz"
tar cvzf $topdir/$debdist ./$DIST
echo "! file \"$debdist\" created."

debsrc="rheolef-$VERSION-source.tgz"
tar cvzf $topdir/$debsrc ./source
echo "! file \"$debsrc\" created."

# clean
cd $topdir; rm -rf $tmpdir


