#ifndef _RHEO_PIOLA_H
#define _RHEO_PIOLA_H
///
/// This file is part of Rheolef.
///
/// Copyright (C) 2000-2009 Pierre Saramito <Pierre.Saramito@imag.fr>
///
/// Rheolef is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.
///
/// Rheolef is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with Rheolef; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
/// 
/// =========================================================================

// Piola transformation:
//  F: hat_K  --> K
//     hat_x |--> x = F(hat_x)
// where F is polynomial Pk
// now just k=1 is supported (no isoparametric transformation yet)
//
// The polynoms of the transformation are evaluated one time
// for all at the nodes of the quadrature formulae on each
// type of reference element: hat_xq in hat_K
//
// author: Pierre Saramito
//
// date: 22 june 2009
//
#include "rheolef/space.h"
#include "rheolef/basis_on_lattice.h"
#include "rheolef/tensor.h"

class piola_on_quadrature : basis_on_quadrature {
public:
  typedef size_t size_type;
  piola_on_quadrature();
  piola_on_quadrature (const quadrature& quad, const space& Xh);
  void set (const quadrature& quad, const space& Xh);

  // F_K (hat_xq)
  point transformation (const geo_element& K, size_type q) const;
  // DF (hat_xq)
  void jacobian_transformation (const geo_element& K, size_type q, tensor& DF) const;
  Float det_jacobian_transformation (const tensor& DF, size_type map_dim) const;
  Float det_jacobian_transformation (const geo_element& K, size_type q) const;


  Float weight_coordinate_system (const geo_element& K, size_type q) const;
  size_type coordinate_dimension () const;
  void set_use_coordinate_system_weight (bool use);
  bool use_coordinate_system_weight () const;
  void set_use_coordinate_system_dual_weight (bool use);
  bool use_coordinate_system_dual_weight () const;
protected:
  space _Xh;
  bool  _use_coordinate_system_weight;
  bool  _use_coordinate_system_dual_weight;
};
#endif // _RHEO_PIOLA_H
