#ifndef _PFSTREAM_H
#define _PFSTREAM_H
///
/// This file is part of Rheolef.
///
/// Copyright (C) 2000-2009 Pierre Saramito <Pierre.Saramito@imag.fr>
///
/// Rheolef is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.
///
/// Rheolef is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with Rheolef; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
/// 
/// =========================================================================

/*Class:pstream
NAME:  @code{ipstream}, @code{opstream} - initiate pipe I/O 
DESCRIPTION:       
 @example
     	ipstream is(COMMAND);
 @end example
 @noindent
 is like the C-style
 @example
	popen(COMMAND, "r");
 @end example
 @noindent
 All input read from @code{is} stream
 is whatever that shell writes to its standard output. E.g.
 @example
 	ipstream is("gzip -dc < results.data.gz")
 @end example

 @noindent
 Conversely,
 @example
 	opstream os(COMMAND);
 @end example
 @noindent
 is like
 @example
	popen(COMMAND, "w");
 @end example
 @noindent
 All output written to stream @code{os} is piped to 
 the standard input of that shell.
 E.g. 
 @example
 	opstream op("gzip -9c > tutu.gz");
 @end example

AUTHORS:
    LMC-IMAG, 38041 Grenoble cedex 9, France
   | Pierre.Saramito@imag.fr
DATE:   31 october 1997
METHODS: @pstream
End:
*/
#include <rheolef/compiler.h>
#include <rheolef/procbuf_c.h>

class procbuf : public filebuf {
  public:
    procbuf();
    procbuf  (const char *name, int mode=ios::in, int prot=0664);
    void open(const char *name, int mode=ios::in, int prot=0664);
    procbuf* close();
    virtual ~procbuf();
    bool is_open() const;
  private:
    procbuf(const procbuf& pbuf, int mode);
    friend class pstream;
  // data
    procbuf_c* _pp;
    int        _mode;
};
//<pstream:
class ipstream : public istream {
  public:
    ipstream();
    ipstream (const char *name, int mode=ios::in, int prot=0664);
    void open(const char *name, int mode=ios::in, int prot=0664);
    void close();
    virtual ~ipstream();
  private:
    void set_pbuf(procbuf* ptr_pbuf);
    procbuf* ptr_pbuf;
};

class opstream : public ostream {
  public:
    opstream();
    opstream(const char *name, int mode=ios::out, int prot=0664);
    void open(const char *name, int mode=ios::out, int prot=0664);
    void close();
    virtual ~opstream();
  private:
    void set_pbuf(procbuf* ptr_pbuf);
    procbuf* ptr_pbuf;
};
//>pstream:

#ifdef TODO
// Does not work yet:
//
// pstream p("COMMAND") is something like: popen("COMMAND", "r+w");
// All input read from ip is whatever that shell writes to its standard output.
// All output written to op is piped to the standard input of that shell.
// E.g. 
//      pstream p("/usr/bin/bc -l");
//      p << "2+2";
//      int result;
//      p >> result;

class pstream : public istream, public ostream {
  public:
    pstream();
    pstream  (const char *name, int mode=ios::in|ios::out, int prot=0664);
    void open(const char *name, int mode=ios::in|ios::out, int prot=0664);
    void close();
    virtual ~pstream();
  private:
    procbuf* ptr_pbuf0;
    procbuf* ptr_pbuf1;
};
#endif // TODO

#endif /*_PFSTREAM_H*/
